// Real Reload v1.3.2 by AngryAmoeba

using GTA;
using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;

public class RealReload : Script {

    Keys ini_ReloadKey;
    bool ini_LoseClipAmmo;

    Ped pc;                                     // The player character
    List<Weapon> gunList = new List<Weapon>();  // List of held guns
    List<int> ammoList = new List<int>();       // List of each gun's last clip ammo
    int currWeaponIndex = 0;                    // The gunList index of the current weapon
    Weapon currWeapon;                          // The current weapon
    int currClip = int.MinValue,                // The current weapon's clip ammo
        extraAmmo = 0,                          // The current weapon's total ammo, minus the clip
        bulletsFired = 0;                       // Used to detect legitimate drops in clip ammo
    bool bReloading = false,                    // Is the player reloading
         bGunIsUsable = false;                  // Can the current weapon be aimed

    public RealReload() {
        Interval = 50;
        Tick += RealReload_Tick;
        KeyDown += RealReload_KeyDown;
        pc = Player.Character;
        currWeapon = pc.Weapons.Current;
        LoadINI();
    }

    private void RealReload_Tick(object sender, EventArgs e) {
        // If the player is reloading...
        if (bReloading) {
            // Cancel reload if player switched weapons before it was complete
            if (currWeapon != pc.Weapons.Current) {
                ammoList[currWeaponIndex] = currClip;
                bReloading = false;
            }
            // Check if reload has been completed (the game has moved ammo into the clip)
            else if (currClip < pc.Weapons.Current.AmmoInClip) {
                // Deduct the discarded clip ammo if the gun uses magazines
                if (ini_LoseClipAmmo && UsesMagazines(currWeapon)) {
                    pc.Weapons.Current.Ammo = pc.Weapons.Current.AmmoInClip = 0;
                    if (extraAmmo > pc.Weapons.Current.MaxAmmoInClip)
                        pc.Weapons.Current.Ammo = extraAmmo - pc.Weapons.Current.MaxAmmoInClip;
                    pc.Weapons.Current.AmmoInClip = ammoList[currWeaponIndex];
                }
                bReloading = false;
            }
        }
        
        // Check whether the current weapon is usable, i.e. can be aimed.
        // Unusable if player just switched weapons. (Also check if clip ammo dropped without
        // firing, a sign that the player switched weapons then switched back really fast.)
        if (currWeapon != pc.Weapons.Current
            || ((currClip > pc.Weapons.Current.AmmoInClip) && bulletsFired == GetBulletsFired()))
            bGunIsUsable = false;
        else
            bulletsFired = GetBulletsFired();
        // Usable once clip ammo is no longer zero (which it's set to while switching weapons)
        if (pc.Weapons.Current.AmmoInClip > 0)
            bGunIsUsable = true;
        
        // *** Don't do anything else until the current weapon is usable
        if (bReloading || !bGunIsUsable)
            return;
        
        // Keep track of current weapon and its clip ammo
        currWeapon = pc.Weapons.Current;
        currClip = pc.Weapons.Current.AmmoInClip;
        
        // If the current weapon is a gun...
        if (IsGun(currWeapon)) {
            // Keep 2 lists: guns used, and how much clip ammo they had last
            if (!gunList.Contains(currWeapon)) {
                gunList.Add(currWeapon);
                ammoList.Add(currClip);
            }
            currWeaponIndex = gunList.IndexOf(currWeapon);
            
            // If clip ammo is different from value in list...
            if (currClip != ammoList[currWeaponIndex]) {
                // Restore list value if clip ammo was refilled without reloading
                if (currClip > ammoList[currWeaponIndex])
                    RevertAmmo();
                // Update list value if clip ammo dropped legitimately
                else
                    ammoList[currWeaponIndex] = currClip;
                
                // Reload if all clip ammo was fired
                if (currClip == 0)
                    ReloadGun();
            }
        }

        // Prune guns that are no longer present
        for (int i = gunList.Count - 1; i >= 0; i--) {
            if (!pc.Weapons.FromType(gunList[i]).isPresent) {
                gunList.Remove(gunList[i]);
                ammoList.Remove(ammoList[i]);
            }
        }
    }

    private void RealReload_KeyDown(object sender, GTA.KeyEventArgs e) {
        if (e.Key == ini_ReloadKey
            && pc.Weapons.Current.AmmoInClip < pc.Weapons.Current.MaxAmmoInClip
            && pc.Weapons.Current.AmmoInClip < pc.Weapons.Current.Ammo
            && pc.Weapons.Current.AmmoInClip > 0
            && IsGun(pc.Weapons.Current))
            ReloadGun();
    }

    // Sets the saved clip ammo to the amount it'll be refilled with
    private void ReloadGun() {
        bReloading = true;
        extraAmmo = pc.Weapons.Current.Ammo - pc.Weapons.Current.AmmoInClip;
        if (extraAmmo >= pc.Weapons.Current.MaxAmmoInClip)
            ammoList[currWeaponIndex] = pc.Weapons.Current.MaxAmmoInClip;
        else
            ammoList[currWeaponIndex] = extraAmmo;
    }

    // Sets the current weapon's clip ammo to the value last saved for it
    private void RevertAmmo() {
        int ammoDiff = pc.Weapons.Current.AmmoInClip - ammoList[currWeaponIndex];
        if (ammoDiff != 0) {
            pc.Weapons.Current.Ammo += ammoDiff;
            pc.Weapons.Current.AmmoInClip -= ammoDiff;
            currClip = pc.Weapons.Current.AmmoInClip;
        }
    }

    private int GetBulletsFired() {
        return Game.GetIntegerStatistic(IntegerStatistic.BULLETS_FIRED);
    }

    private bool IsGun(Weapon weap) {
        if (weap != Weapon.Unarmed
            && weap != Weapon.Misc_Object
            && weap != pc.Weapons.AnyThrown
            && weap != pc.Weapons.AnyMelee)
            return true;
        else
            return false;
    }

    private bool UsesMagazines(Weapon weap) {
        if (weap != pc.Weapons.AnyHeavyWeapon
            && weap != Weapon.Shotgun_Basic
            && weap != Weapon.Shotgun_Baretta
            && weap != Weapon.TLAD_SawedOffShotgun)
            return true;
        else
            return false;
    }
    
    private void LoadINI() {
        SettingsFile ini = SettingsFile.Open(Path.Combine("scripts", "RealReload.ini"));
        ini.Load();
        ini_ReloadKey = ini.GetValueKey("ReloadKey", "REAL RELOAD", Keys.R);
        ini_LoseClipAmmo = ini.GetValueBool("LoseClipAmmo", "REAL RELOAD", true);
    }
} // end class